<?php

namespace Drupal\navigation_blocks\Plugin\Block;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\link\LinkItemInterface;
use Drupal\link\Plugin\Field\FieldWidget\LinkWidget;

/**
 * Provides a 'GenericBackButtonBlock' block.
 *
 * @Block(
 *  id = "generic_back_button_block",
 *  admin_label = @Translation("Generic back button block"),
 * )
 */
class GenericBackButtonBlock extends RefererBackButtonBlock {

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $form['link'] = $this->getLinkFormElements();
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $this->setConfigurationValue('link', $form_state->getValue('link'));
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = parent::build();
    if (!empty($build)) {
      return $build;
    }

    $url = Url::fromUri($this->configuration['link']['url']);
    if ($this->currentRouteMatch->getRouteObject()->getPath() === $url->getInternalPath()) {
      return [];
    }

    return $this->createLink($this->configuration['link']['title'], $url);
  }

  /**
   * Defines the link element of the form array as a tree.
   *
   * @return array
   *   The form.
   */
  private function getLinkFormElements() {
    return [
      '#tree' => TRUE,
      'url' => $this->getUrlElement(),
      'title' => $this->getTextFieldElement(),
    ];
  }

  /**
   * Defines the url element of the form array.
   *
   * @return array
   *   The link element of the form.
   */
  private function getUrlElement () {
    return [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('URL'),
      '#description' => $this->t(
        'Enter the URL you want to generate a back button for.'
      ),
      '#default_value' => $this->getUriAsDisplayableString($this->configuration['link']['url']),
      '#element_validate' => [[LinkWidget::class, 'validateUriElement']],
      '#required' => TRUE,
      '#link_type' => LinkItemInterface::LINK_INTERNAL,
      '#target_type' => 'node',
      '#attributes' => [
        'data-autocomplete-first-character-blacklist' => '/#?',
      ],
      '#process_default_value' => FALSE,
    ];
  }

  /**
   * Defines the url element of the form array.
   *
   * @return array
   *   The linik element of the form.
   */
  private function getTextFieldElement() {
    return [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#description' => $this->t('The title of the back button.'),
      '#default_value' => $this->configuration['link']['title'],
      '#maxlength' => 255,
      '#size' => 64,
    ];
  }

  /**
   * Gets the URI without the 'internal:' or 'entity:' scheme.
   *
   * The following two forms of URIs are transformed:
   * - 'entity:' URIs: to entity autocomplete ("label (entity id)") strings;
   * - 'internal:' URIs: the scheme is stripped.
   *
   * This method is the inverse of ::getUriAsDisplayableString().
   *
   * @param string $uri
   *   The URI to get the displayable string for.
   *
   * @return string
   *
   * @see LinkWidget::getUriAsDisplayableString()
   */
  protected function getUriAsDisplayableString($uri) {
    $scheme = parse_url($uri, PHP_URL_SCHEME);

    // By default, the displayable string is the URI.
    $displayable_string = $uri;

    // A different displayable string may be chosen in case of the 'internal:'
    // or 'entity:' built-in schemes.
    if ($scheme === 'internal') {
      $uri_reference = explode(':', $uri, 2)[1];

      // @todo '<front>' is valid input for BC reasons, may be removed by
      //   https://www.drupal.org/node/2421941
      $path = parse_url($uri, PHP_URL_PATH);
      if ($path === '/') {
        $uri_reference = '<front>' . substr($uri_reference, 1);
      }

      $displayable_string = $uri_reference;
    }
    elseif ($scheme === 'entity') {
      list($entity_type, $entity_id) = explode('/', substr($uri, 7), 2);
      // Show the 'entity:' URI as the entity autocomplete would.
      // @todo Support entity types other than 'node'. Will be fixed in
      //    https://www.drupal.org/node/2423093.
      if ($entity_type === 'node' && $entity = \Drupal::entityTypeManager()->getStorage($entity_type)->load($entity_id)) {
        $displayable_string = EntityAutocomplete::getEntityLabels([$entity]);
      }
    }

    return $displayable_string;
  }

}
