<?php

namespace Drupal\navigation_blocks\Plugin\Block;

use Drupal\Component\Utility\Unicode;
use Drupal\navigation_blocks\BackButtonBlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Url;
use Drupal\navigation_blocks\BlockPathMatcher;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a 'RefererBackButtonBlock' block. The block generates a back button
 * based on the referer and some provided preferred redirects.
 *
 * @Block(
 *  id = "referer_back_button_block",
 *  admin_label = @Translation("Referer back button block"),
 * )
 */
class RefererBackButtonBlock extends BackButtonBlockBase {

  private const BACK_BUTTON_TEXT = 'Back';

  /**
   * The request stack to get the request object from.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Block Path Matcher.
   *
   * @var \Drupal\navigation_blocks\BlockPathMatcher
   */
  protected $blockPathMatcher;

  /**
   * Constructs a new RefererBackButtonBlock object.
   *
   * @param array $configuration
   * @param $pluginId
   * @param $pluginDefinition
   * @param \Drupal\Core\Routing\CurrentRouteMatch $currentRouteMatch
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   * @param \Drupal\navigation_blocks\BlockPathMatcher $blockPathMatcher
   */
  public function __construct(array $configuration, $pluginId, $pluginDefinition, CurrentRouteMatch $currentRouteMatch, RequestStack $requestStack, BlockPathMatcher $blockPathMatcher) {
    parent::__construct($configuration, $pluginId, $pluginDefinition, $currentRouteMatch);
    $this->requestStack = $requestStack;
    $this->blockPathMatcher = $blockPathMatcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $pluginId, $pluginDefinition) {
    return new static(
      $configuration,
      $pluginId,
      $pluginDefinition,
      $container->get('current_route_match'),
      $container->get('request_stack'),
      $container->get('navigation_blocks.path_matcher')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $formState) {
    $form['preferred_redirects'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Preferred redirects'),
      '#description' => $this->t(
        'Please provide the redirects you&#039;d like the back button' .
        ' to refer to. Seperate redirects by a return.'
      ),
      '#default_value' => $this->configuration['preferred_redirects'],
      '#weight' => '0',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $formState) {
    parent::blockSubmit($form, $formState);
    $this->configuration['preferred_redirects'] = $formState->getValue('preferred_redirects');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $refererPath = $this->getRefererPath();
    if (empty($refererPath)) {
      return [];
    }

    $preferredRedirects = $this->getPreferredRedirects();
    if (!$this->blockPathMatcher->matchPath($refererPath, $preferredRedirects)) {
      return [];
    }

    $url = Url::fromUserInput($refererPath);
    return $this->createLink(self::BACK_BUTTON_TEXT, $url);
  }

  /**
   * Get the referer.
   *
   * @return mixed
   *   The referer.
   */
  private function getRefererPath() {
    $headers = $this->requestStack->getCurrentRequest()->headers->all();
    if (!isset($headers['referer'])) {
      return NULL;
    }
    return $this->stripBaseUrl($headers['referer'][0]);
  }

  /**
   * Strips the base URL from the referer.
   *
   * @param string $referer
   *   The referer.
   *
   * @return mixed
   *   The referer with the base URL stripped
   */
  private function stripBaseUrl($referer) {
    global $base_url;
    return str_replace($base_url, '', $referer);
  }

  /**
   * Get the preferred redirects from the configuration and split them by
   * newline.
   *
   * @return string
   *   The preferred redirects
   */
  private function getPreferredRedirects() {
    // Convert path to lowercase. This allows comparison of the same path
    // with different case. Ex: /Page, /page, /PAGE.
    $paths = Unicode::strtolower($this->configuration['preferred_redirects']);
    if (empty($paths)) {
      return '';
    }
    return $paths;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    $cacheContexts = parent::getCacheContexts();
    $cacheContexts[] = 'headers:referer';
    return $cacheContexts;
  }

}
